#!/usr/bin/php
<?php

/*
 * Title: FileView plugin.
 * Version: 1.0.3 (15/Aug/2012)
 * Author: Dmitry.
 * License: GPL.
 * Site: http://usings.ru
*/

@set_time_limit(120);
@error_reporting(E_NONE);
@ini_set('display_errors', 0);

require_once('/usr/local/ispmgr/lib/php/isp_plugin.php');
$plugin = new ISP_Plugin('fileview', PARSER_TYPE_XML);
$xmlParser = $plugin->GetParser();

$xml = $xmlParser->GetXML();
$plugin->LogEvent(print_r($xmlParser->GetInput(), true));
$plugin->LogEvent(print_r($xmlParser->GetParams(), true));

if (!$xml)
	$plugin->Error(1);

$doc = getFirst($xml, 'doc');

$level = $xmlParser->UserLevel;
$plugin->LogEvent('LEVEL: ' . $level);
$user = $xmlParser->UserName;
$plugin->LogEvent('USER: ' . $user);

$func = $xmlParser->GetParam('func');
switch ($func)
{
	case 'fileview':
		$elid = $xmlParser->GetParam('elid');
		$plid = $xmlParser->GetParam('plid');

		// Get user's home dir if needed
		$userdir = '';
		if ($level < 7)
		{
			$passwd = explode("\n", @file_get_contents('/etc/passwd'));
			foreach($passwd as $pass)
				if (strpos($pass, $user . ':') === 0)
				{
					$user_info = explode(':', $pass);
					$userdir = $user_info[5];
					break;
				}
		}
		$plugin->LogEvent('Userdir: ' . $userdir);

		$short_filename = ($plid ? ('/' . $plid) : '') . '/' . $elid;
		appendNode($doc, 'elid', $short_filename);

		$filename = $userdir . $short_filename;
		if (is_dir($filename))
			err(8, $short_filename, 'isdir');
		if (!is_readable($filename))
			err(8, $short_filename, 'unread');
		$mem_limit = pseudoVal(get_cfg_var('memory_limit'));
		$filesize_limit = $mem_limit >= 0 ? min(16 << 20, $mem_limit >> 2) : (16 << 20);
		if (filesize($filename) > $filesize_limit)
			err(8, $short_filename, 'toolarge');
			
		$contents = @file_get_contents($filename);
		$contents = str_replace('&', '&amp;', $contents);
		$contents = remove_ltgt($contents);

		// Change encoding if needed
		$encoding = mb_detect_encoding($contents, 'auto', true);
		if (!$encoding)
			$encoding = 'cp1251';
		else
			$encoding = strtoupper($encoding);
		$plugin->LogEvent('Source encoding: ' . $encoding);
		if ($encoding != 'UTF-8' && $encoding != 'ASCII')
		{
			$contents = iconv($encoding, 'UTF-8', $contents);
			$plugin->LogEvent('Converted to UTF-8');
		}
		$contents = str_replace('', "\xe2\x86\x90", $contents);

		// Check for binary file
		if (strpos($contents, "\0") !== false)
			err(8, $short_filename, 'isbin');

		$contents = htmlspecialchars(
			'<script src="https://yandex.st/highlightjs/6.1/highlight.min.js"></script>' .
			'<script>' .
			'hljs.tabReplace = "    ";' .
			'hljs.initHighlightingOnLoad();' .
			'</script>' .
			'<link rel="stylesheet" href="https://yandex.st/highlightjs/6.1/styles/default.min.css"/>' .
			'<div id="tcont" style="padding:0px; margin-top:1px; overflow-y:auto; overflow-x:auto; max-height:500px; width:100%">' .
			'<table id="t1" style="font-family:courier new; width:100%">' .
			'	<tbody>' .
			'	<tr>' .
			'		<td id="tnums">' .
			'		<pre id="nums" style="padding:0.5em; display:block; text-align:right; vertical-align:middle;"></pre>' .
			'		</td>' .
			'		<td id="tcode" style="width:100%"><pre><code id="code">'
			. $contents .
			'		</code></pre></td>' .
			'	</tr>' .
			'	</tbody>' .
			'</table>' .
			'</div>' .
			'<script type="text/javascript">' .
			'	var nums = document.getElementById("nums");' .
			'	var code = document.getElementById("code");' .
			'	var lineCount = ' . (substr_count($contents, "\n") + 1) . ';' .
			'	var s = "1";' .
			'	for (var i = 2; i <= lineCount; ++i)' .
			'		s = s.concat("<br/>", i);' .
			'	nums.innerHTML = s;' .
			'	var ua = navigator.userAgent.toLowerCase();' .
			'	if (ua.indexOf("msie") != -1)' .
			'		document.getElementById("tcont").style.height = "500px";' .
			'	else if (ua.indexOf("chrome") != -1)' .
			'		document.getElementById("nums").style.marginTop = "-7px";' .
			'</script>');

		$plugin->LogEvent('Output prepared');
		$out = $xml->saveXML();
		echo substr($out, 0, strpos($out, '</doc>')) . '<v>';
		echo $contents;
		echo '</v></doc>';
		$plugin->LogEvent($contents);
		$plugin->LogEvent('Output sent');
		break;

	default:
		err(1);
}

function getFirst($elem, $name)
{
	$list = $elem->getElementsByTagName($name);
	return $list ? $list->item(0) : $list;
}

function appendNode($elem, $name, $value = '')
{
	global $xml;
	$tmp = $xml->createElement($name);
	if ($value)
		$tmp->nodeValue = $value;
	$elem->appendChild($tmp);
	return $tmp;
}

function appendNodes($elem, $array)
{
	foreach ($array as $k => $v)
		appendNode($elem, $k, $v);
}

function remove_ltgt($str)
{
	return str_replace('<', '&lt;', str_replace('>', '&gt;', $str));
}

function pseudoVal($str)
{
	$str = trim($str);
	switch (strtolower(substr($str, -1)))
	{
		case 'k':
			$mul = 1024;
			break;
		case 'm':
			$mul = 1 << 20;
			break;
		case 'g':
			$mul = 1 << 30;
			break;
		default:
			return intval($str);
			break;
	}
	return intval(substr($str, 0, -1)) * $mul;
}

function err($code, $text, $obj = '')
{
	global $xml, $doc;
	$node = appendNode($doc, 'error', $text);
	if ($code)
		$node->setAttribute('code', $code);
	if ($obj)
		$node->setAttribute('obj', $obj);

	$xpath = new DOMXpath($xml);
	$forms = $xpath->query("/doc/metadata/form");
	if ($forms)
	{
		$form = $forms->item(0);
		if ($form)
			$form->removeAttribute('maxwidth');
	}
	echo $xml->saveXML();
	exit;
}
?>
